
import logging

from .current import Current
from .forecast import Forecast
from .kma import Weather

__author__ = 'blurblah (blurblah@blurblah.net)'
__license__ = 'MIT'
__version__ = '0.2.0'

__all__ = (
    'Weather',
    'Current',
    'Forecast'
)

logger = logging.getLogger(__name__)

handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))

logger.addHandler(handler)
logger.setLevel(logging.INFO)




import math


# 아래 링크에 있는 javascript 참고해서 정리한 클래스
# http://www.weather.go.kr/weather/forecast/digital_forecast.jsp?x=60&y=127
class Converter(object):
    RE = 6371.00877 # 지구반경(km)
    GRID = 5.0 # 격자 간격(km)
    SLAT1 = 30.0 # 투영 위도1(degree)
    SLAT2 = 60.0 # 투영 위도2(degree)
    OLON = 126.0 # 기준점 경도(degree)
    OLAT = 38.0 # 기준점 위도(degree)
    XO = 43 # 기준점 X좌표(GRID)
    YO = 136 # 기1준점 Y좌표(GRID)

    def __init__(self):
        self._DEGRAD = math.pi / 180.0
        self._RADDEG = 180.0 / math.pi
        self._base = self._calc_base()

    def _calc_base(self):
        re = self.RE / self.GRID
        _slat1 = self.SLAT1 * self._DEGRAD
        _slat2 = self.SLAT2 * self._DEGRAD
        olon = self.OLON * self._DEGRAD
        olat = self.OLAT * self._DEGRAD

        sn = math.tan(math.pi*0.25 + _slat2*0.5) / math.tan(math.pi*0.25 + _slat1*0.5)
        sn = math.log(math.cos(_slat1) / math.cos(_slat2)) / math.log(sn)
        sf = math.tan(math.pi*0.25 + _slat1*0.5)
        sf = math.pow(sf, sn) * math.cos(_slat1) / sn
        ro = math.tan(math.pi*0.25 + olat*0.5)
        ro = re * sf / math.pow(ro, sn)
        return {
            're': re,
            'olon': olon,
            'sn': sn,
            'sf': sf,
            'ro': ro
        }

    def convert_coord_to_xy(self, lat, lon):
        ra = math.tan((math.pi * 0.25) + (lat * self._DEGRAD * 0.5))
        ra = self._base['re'] * self._base['sf'] / math.pow(ra, self._base['sn'])
        theta = lon * self._DEGRAD - self._base['olon']
        if theta > math.pi:
            theta -= 2.0 * math.pi
        elif theta < -math.pi:
            theta += 2.0 * math.pi
        theta *= self._base['sn']
        return {
            'x': math.floor(ra * math.sin(theta) + self.XO + 0.5),
            'y': math.floor(self._base['ro'] - ra * math.cos(theta) + self.YO + 0.5)
        }

    def convert_xy_to_coord(self, x, y):
        base = self._calc_base()

        xn = x - self.XO
        yn = base['ro'] - y + self.YO
        ra = math.sqrt(xn*xn + yn*yn)
        if base['sn'] < 0.0:
            ra = -ra
        alat = math.pow(base['re'] * base['sf'] / ra, 1.0 / base['sn'])
        alat = 2.0 * math.atan(alat) - math.pi*0.5

        if math.fabs(xn) <= 0.0:
            theta = 0.0
        else:
            if math.fabs(yn) <= 0.0:
                theta = math.pi * 0.5
                if xn < 0.0:
                    theta = -theta
            else:
                theta = math.atan2(xn, yn)
        alon = theta / base['sn'] + base['olon']
        return {
            'lat': alat * self._RADDEG,
            'lon': alon * self._RADDEG
        }


if __name__ == '__main__':
    converter = Converter()
    print(converter.convert_coord_to_xy(37.579871,126.989352))
    print(converter.convert_xy_to_coord(60, 127))




class Current(object):
    def __init__(self, basetime, temperature, humidity, sky, rain_drop=0):
        self.basetime = basetime
        self.temperature = temperature
        self.humidity = humidity
        self.sky = sky  # sky value : 1(clear), 2, 3, 4(cloudy)
        self.rain_drop = rain_drop  # including snow




import certifi
import json
import logging
import urllib3

log = logging.getLogger(__name__)


class IPInfo(object):
    def __init__(self):
        http = urllib3.PoolManager(
            cert_reqs='CERT_REQUIRED',
            ca_certs = certifi.where()
        )
        r = http.request('GET', 'https://ipinfo.io/json')
        self._ipinfo = json.loads(r.data.decode('utf-8'))
        log.debug("Retrieved ip information => {}".format(self._ipinfo))

    def get_location(self):
        location = self._ipinfo['loc'].split(',')
        return {
            'lat': float(location[0]),
            'lon': float(location[1])
        }




import certifi
import json
import logging
import pytz
import urllib3
from datetime import datetime, timedelta
from urllib.parse import unquote

from .current import Current
from .exceptions import KMAException
from .forecast import Forecast
from .ipinfo import IPInfo
from .xy_converter import Converter

log = logging.getLogger(__name__)


class Weather(object):
    DATE_FMT = '%Y%m%d'
    TIME_FMT = '%H%M'
    FORECAST_TIME = ['0200', '0500', '0800', '1100', '1400', '1700', '2000', '2300']
    ENDPOINT = 'http://newsky2.kma.go.kr/service/SecndSrtpdFrcstInfoService2'

    def __init__(self, api_key):
        self._api_key = api_key
        self.http = urllib3.PoolManager(
            cert_reqs='CERT_REQUIRED',
            ca_certs=certifi.where()
        )

    def _calculate_xy_point(self):
        ip_info = IPInfo()
        location = ip_info.get_location()
        return Converter().convert_coord_to_xy(
            location['lat'],
            location['lon']
        )

    def _get_now(self, fmt):
        now = pytz.utc.localize(datetime.utcnow())
        return pytz.timezone('Asia/Seoul') \
            .normalize(now).strftime(fmt)

    def _get_date(self):
        return self._get_now(self.DATE_FMT)

    def _get_time(self):
        return self._get_now(self.TIME_FMT)

    def get_current(self):
        date = self._get_date()
        time = self._get_time()
        date_param = date
        time_param = time[:2] + '00'
        if int(time[2:]) <= 30:
            if int(time[:2]) == 0:
                # yesterday case
                date_param = (datetime.strptime(date, self.DATE_FMT)
                              - timedelta(days=1)).strftime(self.DATE_FMT)
            temp_time = (datetime.strptime(time, self.TIME_FMT)
                         - timedelta(hours=1)).strftime(self.TIME_FMT)
            time_param = temp_time[:2] + '00'
        response = self._request_api('ForecastGrib', date_param, time_param)['response']
        header = response['header']
        if header['resultCode'] == '0000':
            item_list = response['body']['items']['item']
            log.info('Retrieved current weather information.')
            return Current(pytz.timezone('Asia/Seoul')
                           .localize(datetime.strptime(date_param + time_param, '%Y%m%d%H%M')),
                           self._find_current_value(item_list, 'T1H'),
                           self._find_current_value(item_list, 'REH'),
                           self._find_current_value(item_list, 'SKY'),
                           self._find_current_value(item_list, 'RN1'))
        else:
            log.warning('Error response with code:{} and message:{}'
                        .format(header['resultCode'], header['resultMsg']))
            raise KMAException(header['resultCode'], header['resultMsg'])

    def get_forecast(self):
        date = self._get_date()
        time = self._get_time()
        date_param = date
        # 2, 5, 8, 11, 14, 17, 20, 23 hour
        time_index = int(time[:2]) // 3
        time_offset = int(time[:2]) % 3
        if time_offset == 2:
            if int(time[2:]) <= 10:
                time_index -= 1
        else:
            time_index -= 1

        if time_index < 0:
            date_param = (datetime.strptime(date, self.DATE_FMT)
                          - timedelta(days=1)).strftime(self.DATE_FMT)
            time_index = len(self.FORECAST_TIME) - 1
        time_param = self.FORECAST_TIME[time_index]
        response = self._request_api('ForecastSpaceData', date_param, time_param)['response']
        header = response['header']
        if header['resultCode'] == '0000':
            item_list = response['body']['items']['item']
            log.info('Retrieved weather forecast information.')
            return Forecast(pytz.timezone('Asia/Seoul')
                            .localize(datetime.strptime(date_param + time_param, '%Y%m%d%H%M')),
                            self._find_forecast_value(item_list, 'T3H'),
                            self._find_forecast_value(item_list, 'TMN'),
                            self._find_forecast_value(item_list, 'TMX'),
                            self._find_forecast_value(item_list, 'REH'),
                            self._find_forecast_value(item_list, 'POP'))
        else:
            log.warning('Error response with code:{} and message:{}'
                        .format(header['resultCode'], header['resultMsg']))
            raise KMAException(header['resultCode'], header['resultMsg'])

    def _request_api(self, api, base_date, base_time):
        loc = self._calculate_xy_point()
        # service key is encoded and urllib3 request will be encode parameters again
        url = self.ENDPOINT + '/' + api
        log.info('Request url: {}'.format(url))
        log.info('with base_date: {}, base_time: {}, nx: {}, ny: {}, _type: json'
                 .format(base_date, base_time, loc['x'], loc['y']))
        r = self.http.request('GET', url,
                              fields={
                                  'ServiceKey': unquote(self._api_key),
                                  'base_date': base_date,
                                  'base_time': base_time,
                                  'nx': loc['x'],
                                  'ny': loc['y'],
                                  '_type': 'json'
                              })
        json_obj = json.loads(r.data.decode('utf-8'))
        log.debug('Response => ' + json.dumps(json_obj, indent=4))
        return json_obj

    def _find_current_value(self, items, category):
        return self._find_value(items, category, 'obsrValue')

    def _find_forecast_value(self, items, category):
        return self._find_value(items, category, 'fcstValue')

    def _find_value(self, items, category, value_name):
        for item in items:
            if item['category'] == category:
                return item[value_name]
        return None




from setuptools import setup, find_packages

setup(
    name = 'kma',
    version = '0.2.0',
    description = 'Current weather and forecast information library for kma.go.kr',
    long_description=open('README.md').read(),
    url = 'https://github.com/blurblah/kma',
    license='MIT',
    author = 'blurblah',
    author_email = 'blurblah@blurblah.net',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    keywords = 'weather current forecast korea temperature humidity rain kma',
    packages = find_packages(exclude=['tests', 'docs']),
    install_requires = ['urllib3', 'certifi', 'pytz'],
    python_requires = '>=3',
)




class Forecast(object):
    def __init__(self, basetime, temperature_3h,
                 min_temperature, max_temperature,
                 humidity, rain_probability):
        self.basetime = basetime
        self.temperature_3h = temperature_3h
        self.min_temperature = min_temperature
        self.max_temperature = max_temperature
        self.humidity = humidity
        self.rain_probability = rain_probability
